define([ 'require', 'extend', 'events' ], function (require, extend, events) {

    function View(model, controller, arguments) {

        this.model = model;

        this.controller = controller;

        // --- Extensions ---

        // When we instantiate this constructor, we apply to it properties and methods in three steps.
        // Each one of these steps overwrites the methods and properties applied in previous steps.

        // The order of priorities, then, is this: first we apply the default configuration. Then we apply
        // the 'uses' module object, and if a method or property exists in the default configuration
        // that has the same name that one in the 'uses' object, it is overwritten. And then we apply the 
        // 'arguments' object in the same way, overwriting anything that has been applied previously and has
        // the same name as one of the properties or methods of 'arguments'.

        // The difference between the configuration options inside the 'config' object and the stuff you put
        // in the prototype is that the stuff in the prototype is going to be common to every instance while
        // the methods and properties in 'config', though are going to be the same for every instance, are
        // not going to be common: they are really being instantiated although they share value. This is
        // a very subtle difference and maybe there is no sense in having a config object, as we have
        // already a prototype to handle common values. If we want concrete overwrites of those values,
        // we are supposed to specify them on instantiation. So 'config' doesn't have really much sense but,
        // for the time being, I am going to leave it as it is.

        // (Step 1) First we set up a basic common configuraton by default...
        var config = {
            // Mostly empty for the time being ^__^
            id: "",
            tpl: "",
            events: {}
        };

        // ...and we apply this configuration to the current instance that we are creating using my custom method 'extend'
        extend(this, config);

        // (Step 2) Now, if the 'uses' property is present in the arguments, we require that module. Please, don't forget to
        // include that module in the module where you are executing this constructor, or you will get an error.
        this.uses = arguments.uses || [];

        // The method given in the ECMAScript standard to find the class of Object is to use the toString method from Object.prototype
        if (Object.prototype.toString.call( this.uses ) === '[object Array]') { // If you try typeof, the array presents itself as an object
            for (i=0; i<this.uses.length; i++) { 
                extend(this, require(this.uses[i]));
            }
        // If not, we check if it is a string instead
        } else if (typeof this.uses === 'string') {
            extend(this, require(this.uses));
        };

        // (Step 3) Lastly, we apply the arguments object. This is the one defined on instantiation and is supposed to
        // define method and properties that 
        extend(this, arguments);

        // --- Execute initialization functions ---

        // We use an elaborate and highly performant way of testing if something is a function, used by Undescore.js,
        // before we call the actuakl function. I'm not sure if that kind of checking is really necessary, and maybe
        // it can be taken out to improve performance

        // (Step 1) First we executhe the load method, that is supposed to be a previous step before
        // the proper initialization. I'm not really sure if we need this step, probably could be used for loading
        // the ajax spinner.
        if ( !!(this.loading && this.loading.constructor && this.loading.call && this.loading.apply) ) { this.loading() };

        // (Step 2) Now the we execute the init method. Probably the init method contains a call to the 
        // render method, to render the page on the DOM on initialization, but in any case this is not enforced
        // here: the render method is not executed during instantiation by default, as it is possible (though
        // unlikely) that some components' views don't require a render on initialization
        if ( !!(this.init && this.init.constructor && this.init.call && this.init.apply) ) { this.init() };

        if ( !!(this.applyGlobalSubscriptions && this.applyGlobalSubscriptions.constructor && this.applyGlobalSubscriptions.call && this.applyGlobalSubscriptions.apply) ) { this.applyGlobalSubscriptions() };

    }

    View.prototype = {

        // Event handlers

        applyDomListeners: function() {
            // This method applies the event listeners to the DOM. I don't use jQuery on purpose. No specific
            // reason for this, I just wanted to do the framework without other dependencies than Require.js
            //
            //_this and using the call method are needed so the UI functions keep the current object's context
            //
            // _this refers here to the current view. It is necessary because inside each event callback, 'this' points to the 
            // DOM element or to the model firing the event. Invoking the method with call allows us to pass the right context
            // to that method, that is, the current view the method is related to.
            var _this = this;

            function addCrossBrowserEventListener(myElement, myEvent, myFunction) {  
                if (myElement.addEventListener){  // For almost every browser
                    myElement.addEventListener(myEvent, myFunction, false);  
                    return true;  
                } else {  // For IE8 and previous
                    myElement.attachEvent('on'+myEvent, myFunction);  
                    return true;  
                }
            }

            // Here inside the callback, 'this' will refer to the DOM element.
            for(var key in _this.events) {
                if ( key[0] == "#") { // It only works with Ids for the time being. TODO: do it with IDs!
                    addCrossBrowserEventListener( 
                        document.getElementById((key.split(" ")[0]).substring(1)),  
                        key.split(" ")[1], 
                        function(e) {
                            _this[_this.events[key]].call(_this, e) 
                        });
                }
            }; /*document.getElementById("myLink").addEventListener("click", function(e) { _this.confirmation.call(_this, e) });*/
        },

        applyGlobalSubscriptions: function(){
            // This method applies the subscriptions to the glabal events. It is separated from the one above because I think this one is slower and,
            // also, you don't need to call it again after rendering the component for a second time
            var _this = this;
            // Here inside the callback, 'this' will point either to the object that fires the event (probably a model but
            // it can also be another view), or to the global 'Events' object that manages the events.
            for(var key in _this.events) {
                if ( key[0] == "/") { 
                    var functionName = _this.events[key]; 
                    // Careful: do NOT put a reference to the key inside the Events method
                    // If you do, when the event fires it will search for the last value of 'key',
                    // giving you always the last value in the 'events' object. It will always call the last
                    // method on the list. Use 'functionName' instead.
                    Events.subscribe(key, function(args) { _this[functionName].call(_this, args) });
                }
            }; Events.subscribe("/Whatever", function(args) { _this.render.call(_this, args); });
        },

        // Load

        loading: function() {

        },

        // Init

        init: function() {

            // On initialization, first we render the component and, as part of that, we apply the DOM events (take a look
            // inside the 'render' method)
            this.render();

            // Avoid invoking here the method that applies the events, or you will attach the event handlers to the DOM twice: once on init
            // and then again inside the render function! The effect this will have is, for example, that the first click will
            // call the method twice, and that will be very confusing.

        },

        // Render

        render: function() {

            // We render the page and then ready the DOM events
            this.applyDomListeners();

        },

        // Show

        show: function() {

        },

        // Hide

        hide: function() {

        },

        // Destroy

        destroy: function() {

        }

    };

    return View;

});